/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*
 * Project		SHARC L1-L1 using MDMA example - Dual SHARC Talk through.
 * Description  Four channels in, SHARC0 filters two channels while SHARC1 filters
 *              the other two channels in parallel, resulting in four filtered channels.
 * 	            The purpose of this project is to demonstrate using
 * 				two MDMA channels to move raw and filtered data from SHARC0 to SHARC1.
 * 				The project takes four channels of audio from the ADAU1979 CODEC into
 * 				SHARC0 L1 SRAM and sends the raw data to SHARC1 L1 SRAM using MDAM channel A0.
 * 				SHARC0 processes two of the four channels at the time that SHARC1
 * 				is processing the other two channels (parallel processing).
 * 				After SHARC0 completes its processing it uses MDMA channel A1 to
 * 				move the filtered audio channels to SHARC1 L1 SRAM. SHARC1 combines its filtered audio channels
 * 				with the filtered audio channels from SHARC0 resulting in four channels of filtered audio.
 * 				The resulting four channels of filtered audio is sent from SHARC1 L1 SRAM to
 * 				the ADAU1962 DAC driver running on SHARC1. The ARM is used during initialization
 * 				but is not used during runtime.
 *
 *              SHARC0							| SHARC1
 *              ---------------------------------------------------------------
 *			T0	Init ADAU1979 ADC				| Init ADAU1962 DAC
 *			T1	Initialize MDMA channels		| Wait for MDMA init complete
 *			T2	Wait for MDMA interrupt install | Install MDMA interrupt handlers
 *			T3  Enable ADAU1979					| Wait for first audio frame
 *
 *			T4	Send raw audio frame over MDMAA0| MDMA A0 complete callback
 *			T5	Filter audio channels 0,2		| Filter audio channels 1,3  <<<<<<<<<<<<<<<<<<<
 *			T6	Send filtered audio over MDMA A1| MDMA A1 complete callback
 *			T7	Wait for another audio frame	| Merge audio channels, send to ADAU1962 DAC
 *
 * 				--------------   -------------------------------------
 * 				|            |   |     SHARC0 filters channels 0,2   |
 * 				|  ADAU1979  |-->| 4 channels in                     |
 * 				|            |   |                     filtered audio|
 * 				--------------   -------------------------------------
 * 				                      | MDMA A0             | MDMA A1
 * 				                      | raw audio           | 2 channels filtered
 * 				                 ----------------------------------------   ------------
 * 				                 |SHARC1 filters channels 1,3 and merges|   |          |
 *                               |                        4 channels out|-->| ADAU1962 |
 *                               |  SHARC0 chan 0,2 + SHARC1 chan 1,3   |   |          |
 *                               ----------------------------------------   ------------
 *
 *				Griffin's MDMA hardware provides the unique ability to perform data transfer and
 *				signaling in the same package. Data can be transfered directly from SHARC0 L1 to
 *				SHARC1 L1 with both SHARC's getting signaled when the transfer completes. Multiple
 *				MDMA channels are supported and can run concurrently. Each channel provides a unique
 *				signal when the transfer completes.
 */
#include <sys/platform.h>
#include <sys/adi_core.h>
#include <stdio.h>
#include <drivers/adc/adau1979/adi_adau1979.h>
#include "adi_initialize.h"
#include "MultiCoreTalkThruSharcSharc_Core1.h"
#include "SHARC_linkInterface.h"

//*****************************************************************************
// External Functions
//*****************************************************************************
extern uint32_t Adau1979Init(void);
extern uint32_t Adau1979SubmitBuffers(void);
extern uint32_t Adau1979Enable( void );
extern uint32_t Adau1979DoneWithBuffer( void *pBuffer );
extern void SHARC0Filter( void *InputBuffer, void *OutputBuffer, uint32_t BytesToProcess );


//*****************************************************************************
// Local Variables
//*****************************************************************************
uint32_t AdcCount = 0u;
int8_t FilteredData[AUDIO_BUFFER_SIZE];
void *DMASlaveDestinationAddress;

/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

//*****************************************************************************
// SHARC0 core 1
//
// SHARC0 runs the ADAU1979 ADC driver (taking in 4 audio channels),
// sends all four channels to SHARC1 (using MDMA0), filter 2/4 channels, and sends four
// channels (2 filtered, 2 unfiltered) to SHARC1 (using MDMA1).
//*****************************************************************************
int main(int argc, char *argv[])
{
	uint32_t DestAddress;

	//*************************************************************************
	// Initialize managed drivers and/or services that have been added to
	// the project.
	// @return zero on success
	//*************************************************************************
	adi_initComponents();
	
	//*************************************************************************
	// Initialize ADAU1979
	//*************************************************************************
	if(Adau1979Init() != 0u)
	{
		fprintf(stdout, "Core1: Adau1979Init() failed\n" );
		while(1){;}
	}

	if(Adau1979SubmitBuffers() != 0u)
	{
		fprintf(stdout, "Core1: Adau1979SubmitBuffers() failed\n" );
		while(1){;}
	}

	//*************************************************************************
	// Initialize MDMA - Code blocks until both cores complete init
	//*************************************************************************
	if(SHARC_linkMasterInit(&DestAddress) != 0u)
	{
		fprintf(stdout, "Core1: SHARClinkMasterInit() failed\n" );
		while(1){;}
	}

	DMASlaveDestinationAddress = (void *)DestAddress;					// Set address for ADC callback

	*sharc_flag_in_L2 = 0;												// Reset counter

	//*************************************************************************
	// Enable ADAU1979
	//*************************************************************************
	if(Adau1979Enable() != 0u)											// Enable data flow for only the ADC,
	{																	// DAC is enabled only after ping pong buffers of ADC are filled.
		fprintf(stdout, "Core1: Adau1979Enable() failed\n" );
		while(1){;}
	}
	else
		fprintf(stdout, "Core1: ADC Started\n" );

	fprintf(stdout, "Core1: Entering super loop\n" );

	//*************************************************************************
	// Processing is done in the ADC callback context (ISR)
	//*************************************************************************
	while(1){;}

	return 0;
}


//*****************************************************************************
// ADC callback - Called after each audio frame is filled
//				  Audio Format (32 bits/sample) - interleaved (CH0, CH1, CH2, Ch3, ...)
//*****************************************************************************
void AdcCallback(void *pCBParam, uint32_t nEvent, void *pArg)
{
    switch(nEvent)
    {
        case ADI_SPORT_EVENT_RX_BUFFER_PROCESSED:
//        	uint32_t *pTest = pArg;	// Used for testing
//        	for( uint32_t i=0; i<64; ++i )
//        		*pTest++ = i;
        	//*****************************************************************
        	// Send RAW audio to slave SHARC
        	//*****************************************************************
        	if( SHARC_linkSend( MDMA_STREAM_ID_RAW, (void *)pArg, DMASlaveDestinationAddress, 1, AUDIO_BUFFER_SIZE ) != 0 )
        	{
        		while(1){;} // If we get here, there is an error
        	}

        	// Filter the data
        	SHARC0Filter( pArg, FilteredData, AUDIO_BUFFER_SIZE );

        	//*****************************************************************
        	// Send RAW audio to slave SHARC
        	//*****************************************************************
        	if( SHARC_linkSend( MDMA_STREAM_ID_FILTERED, (void *)FilteredData, (DMASlaveDestinationAddress+AUDIO_BUFFER_SIZE), 1, AUDIO_BUFFER_SIZE ) != 0 )
        	{
        		while(1){;} // If we get here, there is an error
        	}

        	*sharc_flag_in_L2 = *sharc_flag_in_L2 + 1;

        	//*****************************************************************
        	// Return buffer to pool
        	//*****************************************************************
        	Adau1979DoneWithBuffer( pArg );
        	AdcCount++;
        	break;
        default:
            break;
    }
}


